<?php

declare(strict_types=1);

namespace Drupal\user_entity_reference_selection\Plugin\EntityReferenceSelection;

use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\ElementInfoManagerInterface;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides common functionality for fixed user selection handlers.
 */
trait UserEntitySelectionTrait {

  /**
   * The element info service.
   */
  protected ElementInfoManagerInterface $elementInfo;

  /**
   * The logger.
   */
  protected LoggerInterface $logger;

  /**
   * Sets the element info service.
   */
  protected function setElementInfo(ElementInfoManagerInterface $element_info): void {
    $this->elementInfo = $element_info;
  }

  /**
   * Sets the logger.
   */
  protected function setLogger(LoggerInterface $logger): void {
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    $default_configuration = ['user_uuid' => User::load(0)?->uuid()];
    return $default_configuration + parent::defaultConfiguration();
  }

  /**
   * Loads a user from the handler's configuration.
   *
   * @param array{user_uuid?: string} $config
   *   The configuration, possibly without defaults merged in.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   */
  protected function getUserFromConfig(array $config, EntityRepositoryInterface $entity_repository): ?UserInterface {
    $user_uuid = $config['user_uuid'] ?? $this->defaultConfiguration()['user_uuid'];
    $user = $entity_repository->loadEntityByUuid('user', $user_uuid);
    if (!$user) {
      $context = [
        '@plugin' => $this->getDerivativeId() ?? $this->getPluginId(),
        '@uuid' => $user_uuid,
      ];
      $this->logger->warning("User entity reference selection handler is configured with user @uuid, but they don't exist.", $context);
    }
    return $user;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);

    $user = $this->getUserFromConfig($this->configuration, $this->entityRepository);
    $form['user_uuid'] = [
      '#title' => $this->t("User to run as"),
      '#type' => 'entity_autocomplete',
      '#target_type' => 'user',
      '#default_value' => $user,
      '#selection_handler' => 'default',
      '#element_validate' => [
        // @phpstan-ignore arrayUnpacking.nonIterable
        ...$this->elementInfo->getInfoProperty('entity_autocomplete', '#element_validate', []),
        [static::class, 'setElementUuid'],
      ],
      '#description' => $this->t("Access checks will be made against this account rather than the current user's. <strong>Be careful if choosing an authenticated user: there are security implications.</strong>"),
    ];

    return $form;
  }

  /**
   * Converts the user value from ID to UUID during element validation.
   *
   * @param mixed[][] $element
   *   The element array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param mixed[][] $form
   *   The form.
   */
  public static function setElementUuid($element, FormStateInterface $form_state, array $form): void {
    $uid = $form_state->getValue($element['#parents']);
    $user = User::load($uid);
    if (!$user) {
      return;
    }
    $form_state->setValueForElement($element, $user->uuid());
  }

  /**
   * {@inheritdoc}
   */
  protected function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS'): QueryInterface {
    $query = parent::buildEntityQuery($match, $match_operator);
    $query->addMetaData('account', $this->currentUser);

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('current_user'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity.repository'),
      $container->get('plugin.manager.element_info'),
      $container->get('logger.channel.user_entity_reference_selection'),
    );
  }

}
